#include <robot_dart/robot_dart_simu.hpp>

#ifdef GRAPHIC
#include <robot_dart/gui/magnum/graphics.hpp>
#endif

/// namespace alias for ease of use
namespace rd = robot_dart;

int main()
{
    /// Create simulator object
    rd::RobotDARTSimu simu(0.001);

    /// Create graphics
#ifdef GRAPHIC
    auto graphics = std::make_shared<rd::gui::magnum::Graphics>();
    simu.set_graphics(graphics);
    graphics->look_at({-2., 3., 2.}, {0., 0., 0.}); // position_of_the_camera, point_to_look_at
#endif

    /// Create world frame
    // size_of_box[x,y,z], pose, "fixed"/"free", mass, color, name
    auto box0 = rd::Robot::create_box(rd::make_vector({0.00001, 0.00001, 0.00001}), Eigen::Vector6d::Zero(), "fixed", 1., rd::make_vector({0., 0., 0., 1.}), "world_frame");
    box0->set_ghost(); // do not simulate dynamics/collisions for this: visual only
    box0->set_cast_shadows(false); // do not cast shadows
    box0->set_draw_axis(box0->body_name(0), 1.);
    simu.add_robot(box0);

    /// Create letter T
    Eigen::Vector3d box1_size;
    box1_size << 0.1, 0.1, 0.5;
    Eigen::Isometry3d tf1 = Eigen::Isometry3d::Identity();
    tf1.linear() = dart::math::eulerZYXToMatrix({M_PI / 3., 0., 0.}); // rotate around Z-axis
    tf1.translation() << 0., 0.5, box1_size[2] / 2.; // translate
    auto box1 = rd::Robot::create_box(box1_size, tf1, "fixed", 1., rd::make_vector({0.9, 0.1, 0.1, 1.}), "box_1");
    box1->set_draw_axis(box1->body_name(0), 0.5);
    simu.add_robot(box1);

    Eigen::Vector3d box2_size;
    box2_size << 0.1, 0.5, 0.1;
    Eigen::Isometry3d tf2 = Eigen::Isometry3d::Identity();
    tf2.linear() = dart::math::eulerZYXToMatrix({M_PI / 3., 0., 0.}); // rotate around Z-axis
    tf2.translation() << 0., 0.5, box1_size[2] + box2_size[2] / 2.; // translate
    auto box2 = rd::Robot::create_box(box2_size, tf2, "fixed", 1., rd::make_vector({0.9, 0.1, 0.1, 1.}), "box_2");
    box2->set_draw_axis(box2->body_name(0), 0.5);
    simu.add_robot(box2);

    /// Create letter Π
    Eigen::Vector3d box3_size;
    box3_size << 0.1, 0.1, 0.5;
    Eigen::Isometry3d tf3 = Eigen::Isometry3d::Identity();
    tf3.translation() << -0.9, 0., box3_size[2] / 2.;
    auto box3 = rd::Robot::create_box(box3_size, tf3, "fixed", 1., rd::make_vector({0.1, 0.2, 0.9, 1.}), "box_3");
    box3->set_draw_axis(box3->body_name(0), 0.5);
    simu.add_robot(box3);

    auto box4 = box3->clone();
    Eigen::Isometry3d tf4 = Eigen::Isometry3d::Identity();
    tf4.translation() << -1.3, 0., box3_size[2] / 2.;
    box4->set_base_pose(tf4);
    box4->set_draw_axis(box4->body_name(0), 0.5);
    simu.add_robot(box4);

    Eigen::Vector3d box5_size = rd::make_vector({0.1, 0.7, 0.1});
    Eigen::Isometry3d tf5 = Eigen::Isometry3d::Identity();
    tf5.linear() = dart::math::eulerZYXToMatrix({M_PI / 2., 0., 0.});
    tf5.translation() << -1.1, 0., box3_size[2] + box5_size[2] / 2.;
    auto box5 = rd::Robot::create_box(box5_size, tf5, "fixed", 1., rd::make_vector({0.1, 0.2, 0.9, 1.}), "box_5");
    box5->set_draw_axis(box5->body_name(0), 0.5);
    simu.add_robot(box5);

    /// Add floor
    simu.add_checkerboard_floor();

    /// Run simulation
    while (true) {
        if (simu.step_world())
            break;
    }

    return 0;
}
