#include <robot_dart/robot_dart_simu.hpp>

#ifdef GRAPHIC
#include <robot_dart/gui/magnum/graphics.hpp>
#endif

/// namespace alias for ease of use
namespace rd = robot_dart;

int main()
{
    /// Create simulator object
    rd::RobotDARTSimu simu(0.001);

    /// Create graphics
#ifdef GRAPHIC
    rd::gui::magnum::GraphicsConfiguration gconfig;
    gconfig.width = 1024;
    gconfig.height = 768;
    auto graphics = std::make_shared<rd::gui::magnum::Graphics>(gconfig);
    simu.set_graphics(graphics);
    graphics->look_at({0., -4., 2.0}, {0., 0., 0.}); // position_of_the_camera, point_to_look_at
    // Record video to "demo.mp4" file
    graphics->camera().record(True);
    graphics->record_video("demo.mp4");
#endif

    /// Create Seesaw
    Eigen::Vector3d box1_size;
    box1_size << 0.1, 0.1, 0.2;
    Eigen::Isometry3d tf1 = Eigen::Isometry3d::Identity();
    tf1.translation() << 0., 0.0, box1_size[2] / 2.;
    auto box1 = rd::Robot::create_box(box1_size, tf1, "fixed", 1., rd::make_vector({0.9, 0.1, 0.1, 1.}), "box_1");
    simu.add_robot(box1);

    Eigen::Vector3d box2_size;
    box2_size << 0.1, 1., 0.05;
    Eigen::Isometry3d tf2 = Eigen::Isometry3d::Identity();
    tf2.linear() = dart::math::eulerZYXToMatrix({M_PI / 2., 0., 0.}); // rotate around Z-axis
    tf2.translation() << 0., 0., box1_size[2] + box2_size[2] / 2.; // translate
    auto box2 = rd::Robot::create_box(box2_size, tf2, "free", 1., rd::make_vector({0.1, 0.9, 0.1, 1.}), "box_2");
    simu.add_robot(box2);

    /// Create sphere
    Eigen::Vector3d sphere_size;
    sphere_size << 0.3, 0.3, 0.3;
    Eigen::Vector6d sphere_pose;
    sphere_pose << 0., 0., 0., 0.4, 0., 5.;
    auto sphere = rd::Robot::create_ellipsoid(sphere_size, sphere_pose, "free", 2., rd::make_vector({0.1, 0.2, 0.9, 1.}), "sphere");
    simu.add_robot(sphere);

    /// Load a robot
    auto robot = std::make_shared<rd::Robot>("pexod.urdf");
    robot->set_position_enforced(true);
    robot->set_actuator_types("servo"); // un-comment this one to have a "dead" hexapod
    Eigen::Isometry3d tf = Eigen::Isometry3d::Identity();
    tf.translation() << -0.3, 0.0, 0.4;
    robot->set_base_pose(tf);
    robot->set_draw_axis(robot->body_name(0), 0.5);
    simu.add_robot(robot);

    /// Add floor
    simu.add_checkerboard_floor();

    /// Run simulation
    while (true) {
        if (simu.step_world())
            break;
    }

    return 0;
}
