#include <robot_dart/robot_dart_simu.hpp>

#ifdef GRAPHIC
#include <robot_dart/gui/magnum/graphics.hpp>
#endif

/// namespace alias for ease of use
namespace rd = robot_dart;

int main()
{
    /// Create simulator object
    rd::RobotDARTSimu simu(0.001);
    // simulate for 20 seconds only
    double simulation_time = 20.0;

    /// Create graphics
#ifdef GRAPHIC
    auto graphics = std::make_shared<rd::gui::magnum::Graphics>();
    simu.set_graphics(graphics);
    graphics->look_at({0., 1.8, 2.}, {0., -0.5, 0.}); // position_of_the_camera, point_to_look_at
#endif

    /// Initial joint position (inside limits)
    double initial_pos = M_PI / 2. - 0.001;

    /// Create 1st robot arm
    auto robot1 = std::make_shared<rd::Robot>("arm.urdf", "robot_1");
    robot1->set_position_enforced(true);
    robot1->set_actuator_types("servo");
    robot1->fix_to_world();
    Eigen::Isometry3d tf = Eigen::Isometry3d::Identity(); // identity transformation -> no translation/rotation
    robot1->set_base_pose(tf);
    robot1->set_positions(rd::make_vector({0., 0., 0., initial_pos})); // Set the initial position of the joints
    auto dofs1 = robot1->dof_names(true, true, true); // Get names of controllable DoFs
    auto robot1_links = robot1->body_names();
    robot1->set_draw_axis(robot1_links.back(), 0.25);
    simu.add_robot(robot1);

    std::cout << "Number of DoFs of robot arm: " << dofs1.size() << std::endl;
    for (auto& s : dofs1)
        std::cout << s << " ";
    std::cout << std::endl;

    /// Create 2nd robot arm
    auto robot2 = std::make_shared<rd::Robot>("arm.urdf", "robot_2");
    robot2->set_position_enforced(true);
    robot2->set_actuator_types("servo");
    robot2->fix_to_world();
    Eigen::Isometry3d tf2 = Eigen::Isometry3d::Identity(); // identity transformation -> no translation/rotation
    tf2.translation() << 0., -0.5, 0.;
    robot2->set_base_pose(tf2);
    robot2->set_positions(rd::make_vector({0., 0., 0., initial_pos})); // Set the initial position of the joints
    auto dofs2 = robot2->dof_names(true, true, true); // Get names of controllable DoFs
    auto robot2_links = robot2->body_names();
    robot2->set_draw_axis(robot2_links.back(), 0.25);
    simu.add_robot(robot2);

    /// Create 2nd robot arm
    auto robot3 = std::make_shared<rd::Robot>("arm.urdf", "robot_3");
    robot3->set_position_enforced(true);
    robot3->set_actuator_types("servo");
    robot3->fix_to_world();
    Eigen::Isometry3d tf3 = Eigen::Isometry3d::Identity(); // identity transformation -> no translation/rotation
    tf3.translation() << 0., -1., 0.;
    robot3->set_base_pose(tf3);
    robot3->set_positions(rd::make_vector({0., 0., 0., initial_pos})); // Set the initial position of the joints
    auto dofs3 = robot2->dof_names(true, true, true); // Get names of controllable DoFs
    auto robot3_links = robot3->body_names();
    robot3->set_draw_axis(robot3_links.back(), 0.25);
    simu.add_robot(robot3);

    /// Handy lambda
    auto round_2_decimals = [](double value) { return std::round(value * 100.) / 100.; };

    /// Controllers (not really :P)
    double previous_position1 = round_2_decimals(robot1->positions()[3]);
    double previous_position2 = round_2_decimals(robot2->positions()[3]);
    double previous_position3 = round_2_decimals(robot3->positions()[3]);
    auto cmds1 = rd::make_vector({0., 0., 0., -1.}); // Velocity commands for each controllable DoF of robot1
    auto cmds2 = rd::make_vector({0., 0., 0., -2.});
    auto cmds3 = rd::make_vector({0., 0., 0., -4.});

    // the commands are NOT reset, no need to set them at every time-step
    robot1->set_commands(cmds1);
    robot2->set_commands(cmds2);
    robot3->set_commands(cmds3);

    /// Add floor
    simu.add_floor();

    /// Run simulation
    while (simu.scheduler().next_time() < simulation_time) {
        if (round_2_decimals(robot1->positions()[3]) == -previous_position1) {
            cmds1[3] = -cmds1[3];
            previous_position1 = round_2_decimals(robot1->positions()[3]);
            robot1->set_commands(cmds1);
        }

        if (round_2_decimals(robot2->positions()[3]) == -previous_position2) {
            cmds2[3] = -cmds2[3];
            previous_position2 = round_2_decimals(robot2->positions()[3]);
            robot2->set_commands(cmds2);
        }

        if (round_2_decimals(robot3->positions()[3]) == -previous_position3) {
            cmds3[3] = -cmds3[3];
            previous_position3 = round_2_decimals(robot3->positions()[3]);
            robot3->set_commands(cmds3);
        }

        if (simu.step_world())
            break;
    }

    return 0;
}
